#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>
#include <unistd.h>
#include <time.h>



void *loadfunc (void)
  {
    char *error;
    void *handle;
    ssize_t (*writefunc) (int fd, const void *buf, size_t count);

    handle = dlopen ("libc.so.6", RTLD_LAZY);
    if (!handle)
      {
        fputs (dlerror (), stderr);
        exit(1);
      }

    writefunc = dlsym(handle, "write");
    if ((error = dlerror ()) != NULL)
      {
        fprintf (stderr, "%s\n", error);
        exit(1);
      }

    return writefunc;
  }


ssize_t
write (int fd, const void *buf, size_t count)
  {
    static ssize_t (*writefunc) (int fd, const void *buf, size_t count) = NULL;

    static int writerate = 0;  /* kB per second */

    static int countsum;
    static long towait = 0;
    static struct timeval writestart;
    static struct timeval writeend;

    struct timespec sleeptime;
    ssize_t ret_val;

    if (!writefunc)
      {
        /* this runs once to get the function from the libc */
        writefunc = loadfunc ();
        if (getenv ("WRITERATE"))
          {
            writerate = atoi (getenv ("WRITERATE"));
          }
      }
    
    if (writerate && !countsum)
      {
        gettimeofday (&writestart, NULL);
      }

    ret_val = (*writefunc) (fd, buf, count);

    if (writerate)
      {
        countsum += count;
        if (countsum > 1024)
          {
            gettimeofday (&writeend, NULL);
            /* count: bytes; writerate: kB/s. */
            /* approximation: 1000000 / 1024 = 977 */
            /* the value 920 is fiddeled to make it more accurate. */
            towait += ( countsum * 920 / writerate ) -
                        ((writeend.tv_sec - writestart.tv_sec) * 1000000 +
                         (writeend.tv_usec - writestart.tv_usec)) ;
            
            if (towait > 100000)
              {
                sleeptime.tv_sec  = (towait * 1000) / 1000000000;
                sleeptime.tv_nsec = (towait * 1000) % 1000000000;
                nanosleep (&sleeptime, NULL);
                towait = 0;
              }
            countsum = 0;
          }
      }

    return ret_val;
  }


